package redditi.model;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;
import java.util.OptionalDouble;

public class CalcolatoreRedditi {
	private Scaglione[] scaglioni;
	private OptionalDouble impostaLorda, impostaNetta;
	private NumberFormat currencyFormatter, percentageFormatter;

	public CalcolatoreRedditi(Scaglione[] scaglioni) {
		currencyFormatter = NumberFormat.getCurrencyInstance(Locale.ITALY);
		currencyFormatter.setMaximumFractionDigits(2);
		currencyFormatter .setMinimumFractionDigits(2);
		percentageFormatter = NumberFormat.getPercentInstance(Locale.ITALY);
		
		this.scaglioni = scaglioni;
		this.impostaLorda= OptionalDouble.empty();
		this.impostaNetta= OptionalDouble.empty();
	}

	public double calcolaImpostaLorda(double reddito, double noTaxArea){
		double imposta=0, imponibile=getRedditoImponibile(reddito,noTaxArea);
		if (imponibile==0) return (this.impostaLorda=OptionalDouble.of(0)).getAsDouble(); 
		int scaglioneMax = determinaIndiceScaglioneMax(imponibile);
		for(int i=scaglioneMax; i>=0; i--){
			Scaglione scaglioneCorrente = scaglioni[i];
				//System.err.println("imponibile attuale: " + imponibile);
				//System.err.println("imposta attuale: " + imposta);
				//System.err.println("scaglione corrente: " + scaglioneCorrente);
			imposta += (imponibile-scaglioneCorrente.getMin())*scaglioneCorrente.getAliquota();
				//System.err.println("imposta aggiornata: " + imposta);
			imponibile = scaglioneCorrente.getMin();
				//System.err.println("imponibile residuo: " + imponibile);
		}
		this.impostaLorda=OptionalDouble.of(imposta); // caching
		return this.impostaLorda.getAsDouble();
	}
	
	public double getRedditoImponibile(double reddito, double noTaxArea){
		return Math.max(0, reddito-noTaxArea);
	}
	
	public double calcolaImpostaNetta(double speseMediche, double aliquotaDetrazione){
		if (!this.impostaLorda.isPresent()) throw new ImpossibleOperationException("imposta lorda non ancora calcolata");
		this.impostaNetta = OptionalDouble.of(this.impostaLorda.getAsDouble()-speseMediche*aliquotaDetrazione);
		// escludiamo volutamente il caso di detrazione enorme che assorbe l'intera imposta, per semplicit
		return this.impostaNetta.getAsDouble();
	}

	public double calcolaImpostaNetta(double speseMediche, String aliquotaDetrazione) throws ParseException{
		return calcolaImpostaNetta(speseMediche, percentageFormatter.parse(aliquotaDetrazione).doubleValue());
	}

	private int determinaIndiceScaglioneMax(double imponibile){
		int i=0;
		while(i < scaglioni.length && 
				scaglioni[i].getMin() < imponibile) i++;
		return i-1;
	}

	public String formatta(String msg, double importo){
		return msg + " " + currencyFormatter.format(importo);
	}

	public double parse(String importo) throws ParseException{
		return this.currencyFormatter.parse(" " + importo).doubleValue();
	}
	
	
/*	public Scaglione determinaScaglione(double imponibile){
		return scaglioni.get(determinaIndiceScaglioneMax(imponibile));
	}
*/

	public Scaglione[] getScaglioni() {
		return scaglioni;
	}
	
	@Override
	public String toString(){
		StringBuilder sb = new StringBuilder();
		for(Scaglione s : scaglioni) sb.append(s.toString() + "\n");
		return sb.toString();
	}
	
	public static void main(String[] args) throws ParseException{
		Scaglione[] scaglioni = new Scaglione[]{
				new Scaglione(0,15000,"10%"),
				new Scaglione(15000,25000,"25%"),
				new Scaglione(25000,45000,"33%"),
				new Scaglione(45000,70000,"38%"),
				new Scaglione(70000,"45%")
		};
		CalcolatoreRedditi calcolatore = new CalcolatoreRedditi(scaglioni);
		System.out.println(calcolatore);
		System.out.println(calcolatore.formatta("Imposta lorda", calcolatore.calcolaImpostaLorda(40000, 8000)));
		System.out.println(calcolatore.formatta("imposta netta", calcolatore.calcolaImpostaNetta(1400,"20%")));
	}

}
